﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;
using System.Globalization;
using System.Collections.ObjectModel;
using System.Configuration;
using System.Diagnostics;

namespace Issue5392
{
    public partial class RemoveVertexForm : Form
    {
        private RotationProjection rotateProjection;

        public RemoveVertexForm()
        {
            InitializeComponent();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            winformsMap1.MapUnit = GeographyUnit.Meter;
            winformsMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.GeographicColors.ShallowOcean);

            ShapeFileFeatureLayer worldLayer = new ShapeFileFeatureLayer(@"..\..\..\..\..\PostsData\Countries02.shp", ShapeFileReadWriteMode.ReadOnly);
            worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1;
            worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            rotateProjection = new RotationProjection();
            worldLayer.FeatureSource.Projection = rotateProjection;


            LayerOverlay worldOverlay = new LayerOverlay();
            worldOverlay.Layers.Add("WorldLayer", worldLayer);
            winformsMap1.Overlays.Add("WorldOverlay", worldOverlay);

            winformsMap1.ZoomLevelSnapping = ZoomLevelSnappingMode.None;
            winformsMap1.CurrentExtent = rotateProjection.GetUpdatedExtent(new RectangleShape(-180.0, 83.0, 180.0, -90.0));

            winformsMap1.Refresh();
        }

        private void btnRotateCounterclockwise_Click(object sender, EventArgs e)
        {
            winformsMap1.Overlays["WorldOverlay"].Lock.EnterWriteLock();
            try
            {
                rotateProjection.Angle += 20;
                winformsMap1.CurrentExtent = rotateProjection.GetUpdatedExtent(winformsMap1.CurrentExtent);
            }
            finally
            {
                winformsMap1.Overlays["WorldOverlay"].Lock.ExitWriteLock();
            }

            //Update the TrackOverlay.
            winformsMap1.TrackOverlay.Lock.EnterWriteLock();
            try
            {
                Collection<Feature> projectedFeatures = new Collection<Feature>();
                double bakRotationAngle = rotateProjection.Angle;
                rotateProjection.Angle = 20;
                foreach (Feature feature in winformsMap1.TrackOverlay.TrackShapeLayer.InternalFeatures)
                {
                    Feature projectedFeature = rotateProjection.ConvertToExternalProjection(feature);
                    projectedFeatures.Add(projectedFeature);
                }
                rotateProjection.Angle = bakRotationAngle;

                winformsMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Clear();

                foreach (Feature feature in projectedFeatures)
                {
                    winformsMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Add(feature);
                }
            }
            finally
            {
                winformsMap1.TrackOverlay.Lock.ExitWriteLock();
            }

            //Update the EditOverlay.
            winformsMap1.EditOverlay.Lock.EnterWriteLock();
            try
            {
                Collection<Feature> projectedFeatures = new Collection<Feature>();
                double bakRotationAngle = rotateProjection.Angle;
                rotateProjection.Angle = 20;
                foreach (Feature feature in winformsMap1.EditOverlay.EditShapesLayer.InternalFeatures)
                {
                    Feature projectedFeature = rotateProjection.ConvertToExternalProjection(feature);
                    projectedFeatures.Add(projectedFeature);
                }
                rotateProjection.Angle = bakRotationAngle;

                winformsMap1.EditOverlay.EditShapesLayer.InternalFeatures.Clear();

                foreach (Feature feature in projectedFeatures)
                {
                    winformsMap1.EditOverlay.EditShapesLayer.InternalFeatures.Add(feature);
                }

                winformsMap1.EditOverlay.CalculateAllControlPoints();
            }
            finally
            {
                winformsMap1.EditOverlay.Lock.ExitWriteLock();
            }

            winformsMap1.Refresh();
        }

        private void btnRotateClockwise_Click(object sender, EventArgs e)
        {
            winformsMap1.Overlays["WorldOverlay"].Lock.EnterWriteLock();
            try
            {
                rotateProjection.Angle -= 20;
                winformsMap1.CurrentExtent = rotateProjection.GetUpdatedExtent(winformsMap1.CurrentExtent);
            }
            finally
            {
                winformsMap1.Overlays["WorldOverlay"].Lock.ExitWriteLock();
            }

            //Update the TrackOverlay.
            winformsMap1.TrackOverlay.Lock.EnterWriteLock();
            try
            {
                Collection<Feature> projectedFeatures = new Collection<Feature>();
                double bakRotationAngle = rotateProjection.Angle;
                rotateProjection.Angle = -20;
                foreach (Feature feature in winformsMap1.TrackOverlay.TrackShapeLayer.InternalFeatures)
                {
                    Feature projectedFeature = rotateProjection.ConvertToExternalProjection(feature);
                    projectedFeatures.Add(projectedFeature);
                }
                rotateProjection.Angle = bakRotationAngle;

                winformsMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Clear();

                foreach (Feature feature in projectedFeatures)
                {
                    winformsMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Add(feature);
                }
            }
            finally
            {
                winformsMap1.TrackOverlay.Lock.ExitWriteLock();
            }

            //Update the EditOverlay.
            winformsMap1.EditOverlay.Lock.EnterWriteLock();
            try
            {
                Collection<Feature> projectedFeatures = new Collection<Feature>();
                double bakRotationAngle = rotateProjection.Angle;
                rotateProjection.Angle = -20;
                foreach (Feature feature in winformsMap1.EditOverlay.EditShapesLayer.InternalFeatures)
                {
                    Feature projectedFeature = rotateProjection.ConvertToExternalProjection(feature);
                    projectedFeatures.Add(projectedFeature);
                }
                rotateProjection.Angle = bakRotationAngle;

                winformsMap1.EditOverlay.EditShapesLayer.InternalFeatures.Clear();

                foreach (Feature feature in projectedFeatures)
                {
                    winformsMap1.EditOverlay.EditShapesLayer.InternalFeatures.Add(feature);
                }

                winformsMap1.EditOverlay.CalculateAllControlPoints();
            }
            finally
            {
                winformsMap1.EditOverlay.Lock.ExitWriteLock();
            }

            winformsMap1.Refresh();
        }

        private void btnEdit_Click(object sender, EventArgs e)
        {
            winformsMap1.TrackOverlay.TrackMode = TrackMode.None;
            winformsMap1.EditOverlay.Lock.EnterWriteLock();
            try
            {
                foreach (Feature feature in winformsMap1.TrackOverlay.TrackShapeLayer.InternalFeatures)
                {
                    winformsMap1.EditOverlay.EditShapesLayer.InternalFeatures.Add(feature);
                }
                winformsMap1.EditOverlay.CalculateAllControlPoints();
            }
            finally
            {
                winformsMap1.EditOverlay.Lock.ExitWriteLock();
            }

            winformsMap1.TrackOverlay.Lock.EnterWriteLock();
            try
            {
                winformsMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Clear();
            }
            finally
            {
                winformsMap1.TrackOverlay.Lock.ExitWriteLock();
            }

            winformsMap1.Refresh();
        }

        private void btnTrackPoint_Click(object sender, EventArgs e)
        {
            winformsMap1.TrackOverlay.TrackMode = TrackMode.Point;
        }

        private void btnTrackLine_Click(object sender, EventArgs e)
        {
            winformsMap1.TrackOverlay.TrackMode = TrackMode.Line;
        }

        private void btnTrackPolygon_Click(object sender, EventArgs e)
        {
            winformsMap1.TrackOverlay.TrackMode = TrackMode.Polygon;
        }

        private void TrackEllipse_Click(object sender, EventArgs e)
        {
            winformsMap1.TrackOverlay.TrackMode = TrackMode.Ellipse;
        }
    }
}
